Object.defineProperty(exports, "__esModule", { value: true });
var gestures_1 = require("@nativescript/core/ui/gestures");
var utils_1 = require("@nativescript/core/utils/utils");
var gesturehandler_1 = require("./gesturehandler");
var gesturehandler_common_1 = require("./gesturehandler.common");
function observe(target, type, callback, context) {
    var observer = new GesturesObserver(target, callback, context);
    observer.observe(type);
    return observer;
}
exports.observe = observe;
var TAG = 0;
var GesturesObserver = (function () {
    function GesturesObserver(target, callback, context) {
        this._target = target;
        this._callback = callback;
        this._context = context;
    }
    Object.defineProperty(GesturesObserver.prototype, "callback", {
        get: function () {
            return this._callback;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GesturesObserver.prototype, "target", {
        get: function () {
            return this._target;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GesturesObserver.prototype, "context", {
        get: function () {
            return this._context;
        },
        enumerable: true,
        configurable: true
    });
    GesturesObserver.prototype.disconnect = function () {
        this._detach();
        if (this.target) {
            this.target.off('loaded', this._onTargetLoaded);
            this.target.off('unloaded', this._onTargetUnloaded);
            this._onTargetLoaded = null;
            this._onTargetUnloaded = null;
        }
        if (this.target) {
            var list = this.target.getGestureObservers(this.type);
            if (list && list.length > 0) {
                for (var i = 0; i < list.length; i++) {
                    if (list[i].callback === this.callback) {
                        break;
                    }
                }
                list.length = 0;
                this.target._gestureObservers[this.type] = undefined;
                delete this.target._gestureObservers[this.type];
            }
        }
        this._target = null;
        this._callback = null;
        this._context = null;
    };
    GesturesObserver.prototype.observe = function (type) {
        var _this = this;
        if (this.target) {
            this.type = type;
            this._onTargetLoaded = function (args) {
                _this._attach(_this.target, type);
            };
            this._onTargetUnloaded = function (args) {
                _this._detach();
            };
            this.target.on('loaded', this._onTargetLoaded);
            this.target.on('unloaded', this._onTargetUnloaded);
            if (this.target.isLoaded) {
                this._attach(this.target, type);
            }
        }
    };
    GesturesObserver.prototype._detach = function () {
        this._notifyTouch = false;
        this._tapGestureHandler = null;
        this._doubletapGestureHandler = null;
        this._longpressGestureHandler = null;
        this._pinchGestureHandler = null;
        this._panGestureHandler = null;
        this._rotationGestureHandler = null;
        this._flingGestureHandler = null;
        this._eventData = null;
    };
    GesturesObserver.prototype.onGestureStateChange = function (type) {
        var _this = this;
        return function (event) {
            if (_this.callback) {
                _this.callback.call(_this._context, __assign(__assign({ eventName: gestures_1.toString(type), object: event.data.view, type: type }, event.data), event.data.extraData));
            }
        };
    };
    GesturesObserver.prototype.onGestureTouchChange = function (type) {
        var _this = this;
        return function (event) {
            if (_this.callback && event.data.state === gesturehandler_common_1.GestureState.ACTIVE) {
                _this.callback.call(_this._context, __assign(__assign({ eventName: gestures_1.toString(type), object: event.data.view, type: type }, event.data), event.data.extraData));
            }
        };
    };
    GesturesObserver.prototype._attach = function (target, type) {
        if (type & gestures_1.GestureTypes.touch) {
            this._notifyTouch = true;
        }
        var manager = gesturehandler_1.Manager.getInstance();
        if (type & gestures_1.GestureTypes.tap) {
            var gestureHandler = (this._tapGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.TAP, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.tap), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.tap), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.longPress) {
            var gestureHandler = (this._longpressGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.LONG_PRESS, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.longPress), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.longPress), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.doubleTap) {
            var gestureHandler = (this._doubletapGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.TAP, TAG++, {
                numberOfTaps: 2,
            }));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.doubleTap), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.doubleTap), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.pinch) {
            var gestureHandler = (this._pinchGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.PINCH, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.pinch), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.pinch), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.swipe) {
            var gestureHandler = (this._flingGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.FLING, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.swipe), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.swipe), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.pan) {
            var gestureHandler = (this._panGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.PAN, TAG++, {}));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.pan), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.pan), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.rotation) {
            var gestureHandler = (this._rotationGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.ROTATION, TAG++, {}));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.rotation), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.rotation), this);
            gestureHandler.attachToView(target);
        }
    };
    GesturesObserver.prototype.androidOnTouchEvent = function (motionEvent) {
        if (this._notifyTouch) {
            if (!this._eventData) {
                this._eventData = new TouchGestureEventData();
            }
            this._eventData.prepare(this.target, motionEvent);
            _executeCallback(this, this._eventData);
        }
    };
    return GesturesObserver;
}());
exports.GesturesObserver = GesturesObserver;
var Pointer = (function () {
    function Pointer(id, event) {
        this.event = event;
        this.ios = undefined;
        this.android = id;
    }
    Pointer.prototype.getX = function () {
        return this.event.getX(this.android) / utils_1.layout.getDisplayDensity();
    };
    Pointer.prototype.getY = function () {
        return this.event.getY(this.android) / utils_1.layout.getDisplayDensity();
    };
    return Pointer;
}());
var TouchGestureEventData = (function () {
    function TouchGestureEventData() {
        this.eventName = gestures_1.toString(gestures_1.GestureTypes.touch);
        this.type = gestures_1.GestureTypes.touch;
        this.ios = undefined;
    }
    TouchGestureEventData.prototype.prepare = function (view, e) {
        this.view = view;
        this.object = view;
        this.android = e;
        this.action = this.getActionType(e);
        this._activePointers = undefined;
        this._allPointers = undefined;
    };
    TouchGestureEventData.prototype.getPointerCount = function () {
        return this.android.getPointerCount();
    };
    TouchGestureEventData.prototype.getActivePointers = function () {
        if (!this._activePointers) {
            this._activePointers = [new Pointer(this.android.getActionIndex(), this.android)];
        }
        return this._activePointers;
    };
    TouchGestureEventData.prototype.getAllPointers = function () {
        if (!this._allPointers) {
            this._allPointers = [];
            for (var i = 0; i < this.getPointerCount(); i++) {
                this._allPointers.push(new Pointer(i, this.android));
            }
        }
        return this._allPointers;
    };
    TouchGestureEventData.prototype.getX = function () {
        return this.getActivePointers()[0].getX();
    };
    TouchGestureEventData.prototype.getY = function () {
        return this.getActivePointers()[0].getY();
    };
    TouchGestureEventData.prototype.getActionType = function (e) {
        switch (e.getActionMasked()) {
            case android.view.MotionEvent.ACTION_DOWN:
            case android.view.MotionEvent.ACTION_POINTER_DOWN:
                return gestures_1.TouchAction.down;
            case android.view.MotionEvent.ACTION_MOVE:
                return gestures_1.TouchAction.move;
            case android.view.MotionEvent.ACTION_UP:
            case android.view.MotionEvent.ACTION_POINTER_UP:
                return gestures_1.TouchAction.up;
            case android.view.MotionEvent.ACTION_CANCEL:
                return gestures_1.TouchAction.cancel;
        }
        return '';
    };
    return TouchGestureEventData;
}());
function _executeCallback(observer, args) {
    if (observer && observer.callback) {
        observer.callback.call(observer._context, args);
    }
}
//# sourceMappingURL=gestures_override.js.map